package org.bluej.extensions.mainrunner;

import bluej.extensions2.BClass;
import bluej.extensions2.BMethod;
import bluej.extensions2.MenuGenerator;
import javafx.scene.control.MenuItem;


import java.lang.reflect.Modifier;

class MenuBuilder extends MenuGenerator {
    
    MenuBuilder(){}

    /**
     * Create the menu item for running the main method, if the latter is available.
     */
    public MenuItem getClassMenuItem(final BClass aClass) {
        MenuItem mi = null;
        if (this.hasMainMethod(aClass)) {
            mi = new MenuItem("Run Main Method");
            mi.setOnAction(event ->  (new Actor(aClass)).run());
        }

        return mi;
    }

    public void notifyPostClassMenu(BClass bc, MenuItem mi) {
        mi.setDisable(!this.hasMainMethod(bc));
    }

    private boolean hasMainMethod(BClass aClass) {
        try {
            if (!aClass.isCompiled()) {
                return false;
            } else {
                // The main method must be exactly signed as expected (public static void main (String[] args),
                // any other coincidental main method signatures would be ignored.
                BMethod bm = aClass.getDeclaredMethod("main", new Class[]{String[].class});
                if (bm == null) {
                    return false;
                } else if (bm.getReturnType() != Void.TYPE) {
                    return false;
                } else {
                    int mod = bm.getModifiers();
                    return Modifier.isPublic(mod) && Modifier.isStatic(mod);
                }
            }
        } catch(Exception ex) {
            return false;
        }
    }
}
